@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2022  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-require File.expand_path('../../test_helper', __FILE__)
+require_relative '../test_helper'
 
 class QueriesControllerTest < Redmine::ControllerTest
   fixtures :projects, :enabled_modules,
@@ -585,6 +585,33 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert q.valid?
   end
 
+  def test_create_admin_projects_query_should_redirect_to_admin_projects
+    @request.session[:user_id] = 1
+
+    q = new_record(ProjectQuery) do
+      post(
+        :create,
+        :params => {
+          :type => 'ProjectQuery',
+          :default_columns => '1',
+          :f => ["status"],
+          :op => {
+            "status" => "="
+          },
+          :v => {
+            "status" => ['1']
+          },
+          :query => {
+            "name" => "test_new_project_public_query", "visibility" => "2"
+          },
+          :admin_projects => 1
+        }
+      )
+    end
+
+    assert_redirected_to :controller => 'admin', :action => 'projects', :query_id => q.id, :admin_projects => 1
+  end
+
   def test_edit_global_public_query
     @request.session[:user_id] = 1
     get(:edit, :params => {:id => 4})
@@ -690,6 +717,33 @@ class QueriesControllerTest < Redmine::ControllerTest
     assert q.valid?
   end
 
+  def test_update_admin_projects_query
+    q = ProjectQuery.create(:name => 'project_query')
+    @request.session[:user_id] = 1
+
+    put(
+      :update,
+      :params => {
+        :id => q.id,
+        :default_columns => '1',
+        :fields => ["status"],
+        :operators => {
+          "status" => "="
+        },
+        :values => {
+          "status" => ['1']
+        },
+        :query => {
+          "name" => "test_project_query_updated", "visibility" => "2"
+        },
+        :admin_projects => 1
+      }
+    )
+
+    assert_redirected_to :controller => 'admin', :action => 'projects', :query_id => q.id, :admin_projects => 1
+    assert Query.find_by_name('test_project_query_updated')
+  end
+
   def test_update_with_failure
     @request.session[:user_id] = 1
     put(
